#!/bin/sh
# Bryant Hansen

ME="$(basename "$0")"
USAGE="$ME [ -v ]"

# default values; these should be loaded from the conf file
CRONTAB=/etc/crontab
CRON_BIN=upload_logfiles.sh

CRON_RANGE_HOURS=12

UPLOAD_LOG=/var/log/logfile_uploader.log

KNOWN_HOSTS_FILE=/home/root/.ssh/known_hosts
DSAPPS_HOST_KEYFILE=/home/root/.ssh/known_hosts_key.txt


DebugMessage() {
    local LOG_RECORD_HEADER="`date "+%Y%m%d_%H%M%S"` (${ME}):"
    echo "${LOG_RECORD_HEADER} $*" >&2
}


CONF_FILE="/etc/logfile_uploader.conf"
if [[ ! -f "$CONF_FILE" ]] ; then
        DebugMessage "ERROR: configuration file $CONF_FILE not found."
	if [[ -f "$PWD"/logfile_uploader.conf ]] ; then
		DebugMessage "installing configuration file from $PWD to $CONF_FILE"
		cp -a "$PWD"/logfile_uploader.conf /etc/
	elif [[ -f "$(dirname "$0")"/logfile_uploader.conf ]] ; then
		DebugMessage "installing configuration file from $(dirname "$0") to $CONF_FILE"
		cp -a "$(dirname "$0")"/logfile_uploader.conf /etc/
	else		
		DebugMessage "ERROR: cannot locate configuration file logfile_uploader.conf"
        DebugMessage "Exiting abnormally."
        exit 2
	fi
fi
. "$CONF_FILE"

VERBOSE=""
if [[ "$1" == "-v" ]] ; then
	VERBOSE=1
	shift
fi

SED_STRIP_COMMENTS="s/\#.*$//"

# verify that we have an entry in the crontab to perform the upload operation
cron_entry="$(cat "$CRONTAB" | sed "$SED_STRIP_COMMENTS" | grep "$CRON_BIN")"
if [[ "$cron_entry" ]] ; then
    DebugMessage "crontab entry is already installed."
    exit 0
fi

# get the PATH that cron uses and set the current PATH to that
cron_path="$(cat "$CRONTAB" | sed "$SED_STRIP_COMMENTS" | grep "PATH=")"
if [[ ! "$cron_path" ]] ; then
    DebugMessage "ERROR: PATH not found in crontab."
    DebugMessage "installation failure"
    exit 3
else
    export "$cron_path"
fi

# verify that the top-level uploader bin is in the cron path
if ! which "$CRON_BIN" > /dev/null 2>&1 ; then
    DebugMessage "ERROR: top-level binary $CRON_BIN not found in PATH."
    DebugMessage "installation failure."
    exit 4
fi

mac=$(get_mac_address.sh)
if [[ ! "$mac" ]] ; then
    DebugMessage "ERROR: failed to retrieve mac address of localhost."
    DebugMessage "  Exiting abnormally."
    exit 5
fi

low_mac=${mac:6}
low_mac_dec=$(printf "%d" "0x$low_mac")
if [[ ! "$low_mac_dec" ]] ; then
    # we sometimes fail here if we don't have a proper mac address
    DebugMessage "ERROR: Failed to convert the low part of the mac address $mac to a number"
    low_mac_dec=0
fi

low_mac_dec_scaled=$(expr $low_mac_dec \* 2)

CRON_RANGE_MIN=$(expr $CRON_RANGE_HOURS \* 60)
low_mac_remainder=$(expr $low_mac_dec_scaled % $CRON_RANGE_MIN)

# pseudo-randomly-select a time
CRON_MIN=$(expr $low_mac_remainder % 60)
CRON_HOUR=$(expr $low_mac_remainder / 60)
CRON_HOUR2=$(expr $CRON_HOUR + 12)

DebugMessage "Installing entry into the crontab (${CRONTAB})"

echo "$CRON_MIN ${CRON_HOUR},${CRON_HOUR2} * * * root $CRON_BIN >> $UPLOAD_LOG 2>&1" >> $CRONTAB
killall -HUP crond

# verify that new entry was installed correctly into the CRONTAB
cron_entry="$(cat "$CRONTAB" | sed "s/\#.*$//" | grep "$CRON_BIN")"
if [[ ! "$cron_entry" ]] ; then
    DebugMessage "ERROR: failed to install crontab entry into $CRONTAB"
    exit 6
fi

exit 0
